#ifndef __PS2QUATERNION_H__
#define __PS2QUATERNION_H__

#include "ps2maths.h"
#include "ps2matrix3x3.h"
#include "ps2matrix4x4.h"
#include "ps2vector3.h"

namespace KazMaths
{
	class Quaternion
	{
	public:
		Quaternion(void);
		Quaternion(float _x, float _y, float _z, float _w);
		~Quaternion(void);

		Quaternion & NormaliseSelf();
		Matrix3x3 ToRotationMatrix();
		Matrix4x4 ToRotationMatrix4x4();

		void EulerAngles(float roll, float pitch, float yaw);

		float w,x,y,z;
	};
}

inline KazMaths::Quaternion operator + (const KazMaths::Quaternion &q1,
										const KazMaths::Quaternion &q2)
{
	return KazMaths::Quaternion(q1.x + q2.x, q1.y + q2.y, q1.z + q2.z, q1.w + q2.w);
}

inline KazMaths::Quaternion operator - (const KazMaths::Quaternion &q1,
										const KazMaths::Quaternion &q2)
{
	return KazMaths::Quaternion(q1.x - q2.x, q1.y - q2.y, q1.z - q2.z, q1.w - q2.w);
}

inline KazMaths::Quaternion operator * (const KazMaths::Quaternion &q1,
										const KazMaths::Quaternion &q2)
{
	// Qr = Q1.Q2 = ( w1.w2 - v1.v2, w1.v2 + w2.v1 + v1 x v2 )
	KazMaths::Vector3 v1(q1.x,q1.y,q1.z),v2(q2.x,q2.y,q2.z);
	KazMaths::Vector3 ret;

	ret = q1.w * v2 + q2.w * v1 + CrossProduct(v2,v1);

	return KazMaths::Quaternion(ret.x,ret.y,ret.z,q1.w * q2.w - DotProduct(v1,v2));

}

inline KazMaths::Quaternion operator * (const KazMaths::Quaternion &q,
										const float &s)
{
	return KazMaths::Quaternion(q.x * s, q.y * s, q.z * s, q.w * s);
}

inline KazMaths::Quaternion operator * (const float &s,
										const KazMaths::Quaternion &q)
{
	return KazMaths::Quaternion(q.x * s, q.y * s, q.z * s, q.w * s);
}

#endif
