#ifndef __PS2VECTOR3_H__
#define __PS2VECTOR3_H__

#include "ps2maths.h"

namespace KazMaths
{
	class Vector3
	{
	public:
		// Constructors
		Vector3(void);
		Vector3(const float _x, const float _y, const float _z);
		Vector3(const Vector3 & rhs);

		~Vector3(void);

		// Operations with other vectors
		Vector3 & operator+=(const Vector3 & rhs);
		Vector3 & operator-=(const Vector3 & rhs);

		// Special arithmetic
		float Dot(const Vector3 & rhs) const;
		Vector3 Cross(const Vector3 & rhs) const;

		Vector3 & operator*=(const float s);
		Vector3 & operator/=(const float s);

		bool operator==(const Vector3 & rhs) const;

		// Miscellaneous
		float Length() const;
		float LengthSqr() const;
		Vector3 Normalise();
		void NormaliseSelf();

		// Vector projection
		Vector3 ProjectOnto(const Vector3 & rhs) const;
		Vector3 PerpendicularOnto(const Vector3 & rhs) const;
		
		// Member data
		float x,y,z; 
	};
}

inline KazMaths::Vector3 operator + (const KazMaths::Vector3 &v1,
									 const KazMaths::Vector3 &v2)
{
	return KazMaths::Vector3(v1.x + v2.x, v1.y + v2.y, v1.z + v2.z);
}

inline KazMaths::Vector3 operator - (const KazMaths::Vector3 &v1,
									 const KazMaths::Vector3 &v2)
{
	return KazMaths::Vector3(v1.x - v2.x, v1.y - v2.y, v1.z - v2.z);
}

inline KazMaths::Vector3 operator - (const KazMaths::Vector3 &v1)
{
	return KazMaths::Vector3(-v1.x, -v1.y, -v1.z);
}

inline KazMaths::Vector3 operator * (const KazMaths::Vector3 &v,
									 const float &s)
{
	return KazMaths::Vector3(v.x * s, v.y * s, v.z * s);
}

inline KazMaths::Vector3 operator * (const float & s,
									 const KazMaths::Vector3 &v)
{
	return KazMaths::Vector3(v.x * s, v.y * s, v.z * s);
}

inline KazMaths::Vector3 operator / (const KazMaths::Vector3 &v,
									 const float & s)
{
	return KazMaths::Vector3(v.x / s, v.y / s, v.z / s);
}

inline KazMaths::Vector3 CrossProduct (const KazMaths::Vector3 &v1,
									   const KazMaths::Vector3 &v2)
{
	return KazMaths::Vector3(v1.y * v2.z - v1.z * v2.y,
							 v1.z * v2.x - v1.x * v2.z,
							 v1.x * v2.y - v1.y * v2.x);
}

inline float DotProduct (const KazMaths::Vector3 &v1,
						 const KazMaths::Vector3 &v2)
{
	return (v1.x * v2.x + v1.y * v2.y + v1.z * v2.z);
}

inline KazMaths::Vector3 Normalise (const KazMaths::Vector3 &v)
{
	return v / v.Length();
}

#endif
